#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from   .mumamisc import *

class CheckPlaylist(wx.Dialog):
    ID_SKIP = wx.NewId()
    ID_START = wx.NewId()
    ID_CB_DB = wx.NewId()
    ID_CB_HD = wx.NewId()
    
    def __init__(self, parent, config, mumadb, playlists, onCheckMusicDir, onPlayTitle):
        windowpos = (wx.Point(int(config['Window']['cplposx']),
                              int(config['Window']['cplposy'])))
        windowsize = (wx.Size(int(config['Window']['cplsizex']),
                              int(config['Window']['cplsizey'])))               
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - Check playlist',
                           pos = windowpos, size = windowsize,
                           style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.Bind(wx.EVT_CLOSE, self.onButtonClose)
        self.checkrunning = False
        self.closerequest = False
        self.parent = parent
        self.config = config
        self.mumadb = mumadb
        self.playlists = playlists
        self.onCheckMusicDir = onCheckMusicDir
        self.onPlayTitle = onPlayTitle
        btnadj = int(config['Misc']['norm'] == 'False')
        self.selplname = config['View']['selplname']        
        if len(self.selplname):
            self.playlist = self.playlists[self.selplname]
        else:
            self.playlist = []
        self.playlen = len(self.playlist)

        sizer = wx.BoxSizer(wx.VERTICAL)
        row1 = wx.BoxSizer(wx.HORIZONTAL)         
        sb = wx.StaticBox(self, -1, " Check playlist '" + self.selplname + "'")
        col1 = wx.StaticBoxSizer(sb, wx.HORIZONTAL)
        self.tc = wx.TextCtrl(self, -1, style = wx.TE_RICH | wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_DONTWRAP)      
        if LINUX and wx.VERSION_STRING < '3': self.tc.Enable(False)  
        background_color = wx.SystemSettings.GetColour(READONLY_COLOR)
        self.tc.SetBackgroundColour(background_color)
        col1.Add(self.tc, 1, wx.GROW | wx.ALL, GAP)
        row1.Add(col1, 1, LRT | wx.GROW, GAP)

        sb = wx.StaticBox(self, -1, ' Title alternatives ')
        col2 = wx.StaticBoxSizer(sb, wx.HORIZONTAL)
        self.ti = wx.ListCtrl(self, -1, style = wx.LC_REPORT | \
                              wx.LC_SINGLE_SEL | wx.LC_HRULES | wx.BORDER_SUNKEN)
        imagelist = wx.ImageList(16, 16)
        #imagelist.Add(wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, wx.Size(16, 16)))
        imagelist.Add(loadIcon('audiofile.png'))
        self.ti.AssignImageList(imagelist, wx.IMAGE_LIST_SMALL)        
        self.ti.InsertColumn(0, 'Id_Title')
        self.ti.InsertColumn(1, 'Title')
        self.ti.InsertColumn(2, 'Interpreter')
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.onItemSelected, self.ti)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onPlayItemClicked, self.ti)
        col2.Add(self.ti, 1, wx.GROW | wx.ALL, GAP)
        row1.Add(col2, 1, LRT | wx.GROW, GAP)
              
        sb = wx.StaticBox(self, -1, ' Check targets ')
        row2 = wx.StaticBoxSizer(sb, wx.HORIZONTAL)
        self.cbdb = wx.CheckBox(self, self.ID_CB_DB, ' check database entries') 
        row2.Add(self.cbdb, 0, wx.ALL, GAP)
        row2.AddStretchSpacer()
        self.cbhd = wx.CheckBox(self, self.ID_CB_HD, ' check music device files') 
        row2.Add(self.cbhd, 0, wx.ALL, GAP)
        row2.AddStretchSpacer()
        self.cbdb.SetValue(True)
        self.cbhd.SetValue(onCheckMusicDir())
        self.Bind(wx.EVT_CHECKBOX, self.onCheckMusicDevice, id = self.ID_CB_HD)
        self.butskip = wx.Button(self, self.ID_SKIP, 'S&kip')
        self.butpaste = wx.Button(self, wx.ID_PASTE)
        butclose = wx.Button(self, wx.ID_CLOSE)
        self.butstart = wx.Button(self, self.ID_START, '&Start')
        self.Bind(wx.EVT_BUTTON, self.onButtonClose, id = wx.ID_CLOSE)
        self.Bind(wx.EVT_BUTTON, self.onButtonPaste, id = wx.ID_PASTE)
        self.Bind(wx.EVT_BUTTON, self.onButtonSkip, id = self.ID_SKIP)
        self.Bind(wx.EVT_BUTTON, self.onButtonStart, id = self.ID_START)
        row3 = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: row3.AddStretchSpacer()
        row3.Add(self.butstart, btnadj, wx.GROW | wx.RIGHT, GAP)
        row3.Add(self.butskip, btnadj, wx.GROW | wx.LEFT | wx.RIGHT, GAP)
        row3.Add(self.butpaste, btnadj, wx.GROW | wx.LEFT | wx.RIGHT, GAP)
        row3.Add(butclose, btnadj, wx.GROW | wx.LEFT, GAP)

        sizer.Add(row1, 1, wx.GROW | LRT, GAP)
        sizer.Add(row2, 0, wx.GROW | LRT, BORDER)
        sizer.Add(row3, 0, wx.GROW | wx.ALL, BORDER)
        self.SetSizer(sizer)
        self.SetMinSize(wx.Size(800, 520))
        #self.Fit()
        #self.Centre()
        self.disableButtons()
    #-------------------------------------------------------------------------------------------

    def onCheckMusicDevice(self, event):
        if self.cbhd.IsChecked() and not self.onCheckMusicDir():
            self.cbhd.SetValue(False)
            dlg = wx.MessageDialog(self, "The music device is not mounted / reachable!", TITLE + ' - Check music device', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
    #-------------------------------------------------------------------------------------------

    def onItemSelected(self, event):
        self.selectindex = event.GetIndex() 
    #-------------------------------------------------------------------------------------------

    def onPlayItemClicked(self, event):
        if not self.onCheckMusicDir():
            self.cbhd.SetValue(False)
            dlg = wx.MessageDialog(self, "The music device is not mounted / reachable!", TITLE + ' - Play title', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        
        index = event.GetIndex() 
        item = self.ti.GetItem(index, 0)
        id_title = item.GetText()
        self.onPlayTitle(id = id_title)
    #-------------------------------------------------------------------------------------------

    def disableButtons(self):
        self.butskip.Disable()
        self.butpaste.Disable()
    #-------------------------------------------------------------------------------------------

    def enableButtons(self):
        self.butskip.Enable()
        self.butpaste.Enable()
    #-------------------------------------------------------------------------------------------
        
    def onButtonStart(self, event):
        self.index = 0
        self.selectindex = -1
        self.tc.Clear()
        self.butstart.Disable()
        self.doTheCheck()
    #-------------------------------------------------------------------------------------------

    def windowPosition(self):
        windowrect = self.GetRect()
        self.config['Window']['cplposx'] = windowrect[0]
        self.config['Window']['cplposy'] = windowrect[1]
        self.config['Window']['cplsizex'] = windowrect[2]
        self.config['Window']['cplsizey'] = windowrect[3]
    #-------------------------------------------------------------------------------------------

    def onButtonClose(self, event):
        if self.checkrunning:
            self.closerequest = True
        else:
            self.windowPosition()
            self.Destroy()
    #-------------------------------------------------------------------------------------------
        
    def onButtonSkip(self, event):
        self.doTheCheck()
    #-------------------------------------------------------------------------------------------

    def onButtonPaste(self, event):
        if self.selectindex < 0: return
        item = self.ti.GetItem(self.selectindex, 0)
        id_title = item.GetText()
        item = self.ti.GetItem(self.selectindex, 1)
        title = item.GetText()
        index = self.index - 2
        self.playlist[index] = title
        self.playlist[index + 1] = id_title
        self.doTheCheck()
    #-------------------------------------------------------------------------------------------

    def logText(self, text, color = None):
        if color == None: color = BLACK
        start = self.tc.GetInsertionPoint()
        self.tc.WriteText('%s\n' % text)
        stop = self.tc.GetLastPosition()
        self.tc.SetStyle(start, stop, wx.TextAttr(color))
        wx.SafeYield(self)
        self.tc.ShowPosition(stop)                   
    #-------------------------------------------------------------------------------------------

    def findAlternatives(self, title):
        sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Title LIKE "%%%s%%"' % (title)
        matches, count = self.mumadb.getMatches(sql)
        for index, match in enumerate(matches):
            self.ti.InsertImageStringItem(index, str(match[0]), 0)
            self.ti.SetStringItem(index, 1, match[1])
            self.ti.SetStringItem(index, 2, match[2])
        self.ti.SetColumnWidth(0, wx.LIST_AUTOSIZE)
        if not LINUX: self.ti.SetColumnWidth(0, self.ti.GetColumnWidth(0) + 5)
        self.ti.SetColumnWidth(1, wx.LIST_AUTOSIZE)
        self.ti.SetColumnWidth(2, wx.LIST_AUTOSIZE)
        return count   
    #-------------------------------------------------------------------------------------------

    def doTheCheck(self):
        checkdb = self.cbdb.IsChecked()
        if not checkdb: count = 1
        checkhd = self.cbhd.IsChecked()
        if not checkhd: found = True
        if not checkdb and not checkhd:
            self.butstart.Enable()
            return
        self.checkrunning = True
        self.disableButtons()
        self.ti.DeleteAllItems()
        altcount = 0
        self.selectindex = -1
        while not self.closerequest and self.index < self.playlen:
            title = self.playlist[self.index]
            id_title = self.playlist[self.index + 1]
            self.index += 2
            if checkhd:
                found, pathname = findTitleExt(self.config, id_title)
            if checkdb:
                sql = 'SELECT Id_Title FROM muma_title WHERE Id_Title = "%s"' % (id_title)
                matches, count = self.mumadb.getMatches(sql)
            if found and count == 1:
                self.logText('%s, %s - checked ok' % (id_title, title))
                altcount = 0
            else:   
                self.logText('%s, %s - not found' % (id_title, title), RED)
                altcount = self.findAlternatives(title)
                if altcount:
                    self.logText('select one title alternative and press paste or press skip')
                    self.enableButtons()
                    break

        if self.index >= self.playlen and not altcount:
            self.logText("\nchecking playlist '%s' done ..." % self.selplname)
            self.disableButtons()
            self.butstart.Enable()

        self.checkrunning = False
        if self.closerequest:
            self.windowPosition()
            self.Destroy()
# - class CheckPlaylist(wx.Dialog):--------------------------------------------------------------

            