#!/usr/bin/env python
#-*- coding: UTF-8 -*-
 
import wx
import os.path
from   .mumamisc import *

class DirDialog(wx.Dialog):
    
    ID_CB_HIDDEN = wx.NewId()
    ID_TEXTCTRL =  wx.NewId()
    
    def __init__(self, parent, title, config, pname = '/', fname = '', mode = 'opend', filters = ''):        
        """ Manage dirs dialog
        mode is one of: copy, openf, opend, save """
        windowpos = (wx.Point(int(config['Window']['dirposx']),
                              int(config['Window']['dirposy'])))
        windowsize = (wx.Size(int(config['Window']['dirsizex']),
                              int(config['Window']['dirsizey'])))                
        wx.Dialog.__init__(self, parent, -1, title, pos = windowpos, size = windowsize,                          
                           style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.config = config
        self.hidden = config['Misc']['hiddendirs'] == 'True'
        btnadj = int(config['Misc']['norm'] == 'False')
        if pname == '': pname = '/'
        self.mode = mode
        self.oldpath = pname
        self.fname = fname
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        if mode in ['openf', 'save'] and filters != None:
            sb = wx.StaticBox(self, -1, ' Select file ')
            style = wx.DIRCTRL_SHOW_FILTERS | wx.SUNKEN_BORDER
        else:
            sb = wx.StaticBox(self, -1, ' Select dir ')
            style = wx.DIRCTRL_DIR_ONLY | wx.SUNKEN_BORDER       
        dsizer = wx.StaticBoxSizer(sb, wx.VERTICAL)
        self.dirctrl = wx.GenericDirCtrl(self, size = wx.Size(200, 200), style = style, filter = filters)
                                             # size = wx.DefaultSize generates assertion - 200, 200 ist ignored 
        self.dirctrl.ShowHidden(self.hidden)
        style = self.dirctrl.GetTreeCtrl().GetWindowStyle() | wx.TR_FULL_ROW_HIGHLIGHT
        self.dirctrl.GetTreeCtrl().SetWindowStyle(style)        
        self.dirctrl.SetPath(pname)
        dsizer.Add(self.dirctrl, 1, wx.GROW | wx.ALL, GAP)
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.onSelChange)
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.onActivated)
                        
        tcsizer = wx.BoxSizer(wx.HORIZONTAL)
        tcsizer.Add(wx.StaticText(self, -1, 'Path: '), 0)
        pname = self.dirctrl.GetPath()
        self.pname = pname
        self.statctrl = wx.StaticText(self, -1, pname + ' ')
        tcsizer.Add(self.statctrl, 0)
        self.dynctrl = wx.TextCtrl(self, self.ID_TEXTCTRL, fname, style = wx.TE_PROCESS_ENTER)            
        tcsizer.Add(self.dynctrl, 1)
        if mode in ['copy', 'save']:
            self.statctrl.SetLabel(pname + os.sep + ' ')
            self.dynctrl.SetValue(fname) 
        if mode in ['copy', 'opend', 'openf']:
            self.dynctrl.SetBackgroundColour(wx.SystemSettings.GetColour(READONLY_COLOR))
            self.dynctrl.SetEditable(False)                     
        if mode == 'save':
                self.hitreturn = wx.StaticText(self, -1, 'select or enter a file name')
        else:    
            self.hitreturn = wx.StaticText(self, -1, '')
        font = wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        self.hitreturn.SetFont(font)
        dsizer.Add(self.hitreturn, 0, wx.GROW | wx.LEFT | wx.RIGHT, GAP)
        #self.Bind(wx.EVT_TEXT_ENTER, self.onTextEnter, id = self.ID_TEXTCTRL)
        dsizer.Add(tcsizer, 0, wx.GROW | wx.ALL, GAP)

        ccsizer = wx.BoxSizer(wx.HORIZONTAL)        
        self.cbh = wx.CheckBox(self, self.ID_CB_HIDDEN, ' show hidden / dotted dirs') 
        ccsizer.Add(self.cbh, 0, wx.LEFT, 0)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckHidden)
        self.cbh.SetValue(self.hidden)
        ccsizer.AddStretchSpacer()
        butnew = wx.Button(self, wx.ID_NEW, 'New Folder')
        self.Bind(wx.EVT_BUTTON, self.onButtonNew, id = wx.ID_NEW)   
        ccsizer.Add(butnew, 0, wx.GROW | wx.LEFT, BORDER)       
        dsizer.Add(ccsizer, 0, wx.GROW | wx.ALL, GAP)        
        
        sizer.Add(dsizer, 1, LRT | wx.GROW, BORDER)  
        if mode == 'save':
            butcos = wx.Button(self, wx.ID_SAVE)
            self.Bind(wx.EVT_BUTTON, self.onButtonSave, id = wx.ID_SAVE)
            self.Bind(wx.EVT_TEXT_ENTER, self.onButtonSave, id = self.ID_TEXTCTRL)
        elif mode in ['opend', 'openf']:
            butcos = wx.Button(self, wx.ID_OPEN)
            self.Bind(wx.EVT_BUTTON, self.onButtonCopy, id = wx.ID_OPEN)
        else: # mode == copy
            self.musicformat = ['mp3           ', 'ogg           ', 'wav           ', 'do not change']
            rbe = wx.RadioBox(self, -1, ' Convert musicfile to format ', wx.DefaultPosition, wx.DefaultSize,
                              self.musicformat, 1, wx.RA_SPECIFY_ROWS)
            sizer.Add(rbe, 0, LRT | wx.GROW, BORDER)        
            self.Bind(wx.EVT_RADIOBOX, self.onRadioMusicFormat, rbe)
            rbe.SetSelection(int(self.config['Misc']['musicformat']))
            self.fileformat = ['ID_Title.ext ', 'Interpreter-Title.ext ', 'nn-Interpreter-Title.ext']
            rbf = wx.RadioBox(self, -1, ' Select target filename format ', wx.DefaultPosition, wx.DefaultSize,
                             self.fileformat, 1, wx.RA_SPECIFY_ROWS)
            sizer.Add(rbf, 0, LRT| wx.GROW, BORDER)        
            self.Bind(wx.EVT_RADIOBOX, self.onRadioFileFormat, rbf)
            rbf.SetSelection(int(self.config['Misc']['fileformat']))
            butcos = wx.Button(self, wx.ID_COPY)
            self.Bind(wx.EVT_BUTTON, self.onButtonCopy, id = wx.ID_COPY)  
 
        butcancel = wx.Button(self, wx.ID_CANCEL)    
        bs = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: bs.AddStretchSpacer()
        bs.Add(butcos, btnadj, wx.GROW, BORDER)
        bs.Add(butcancel, btnadj, wx.GROW | wx.LEFT, BORDER)
        sizer.Add(bs, 0,  wx.GROW | wx.CENTER | wx.ALL, BORDER)
        
        self.SetSizer(sizer)
        self.SetMinSize(wx.Size(450, 450))
        self.newpressed = False
    #-------------------------------------------------------------------------------------------

    def onRadioFileFormat(self, event):
        self.config['Misc']['fileformat'] = str(event.GetInt())
    #-------------------------------------------------------------------------------------------

    def onRadioMusicFormat(self, event):
        self.config['Misc']['musicformat'] = str(event.GetInt())       
    #-------------------------------------------------------------------------------------------

    def onCheckHidden(self, event):
        self.hidden = not self.hidden
        self.dirctrl.ShowHidden(self.hidden)        
    #-------------------------------------------------------------------------------------------

    def onSelChange(self, event):
        self.newpressed = False
        self.hitreturn.SetLabel('')
        self.dynctrl.SetEditable(False)
        self.dynctrl.SetBackgroundColour(wx.SystemSettings.GetColour(READONLY_COLOR))
        
        pname = self.dirctrl.GetPath()
        if self.mode == 'save':
            self.hitreturn.SetLabel('select or enter a file name')
            self.dynctrl.SetEditable(True)
            self.dynctrl.SetBackgroundColour(wx.NullColour)
            if os.path.isfile(pname):
                dpart, fpart = os.path.split(pname)
                self.statctrl.SetLabel(dpart + os.sep + ' ')
                self.dynctrl.SetValue(fpart)
                self.pname = dpart
                self.fname = fpart
            else:    
                self.statctrl.SetLabel(pname + os.sep + ' ')
                self.dynctrl.SetValue(self.fname)
                self.pname = pname
        else:
            self.statctrl.SetLabel(pname + ' ')
            if self.mode == 'copy':
                self.pname = pname
        self.oldpath = pname
        self.Layout()
    #-------------------------------------------------------------------------------------------
    
    def onActivated(self, event):
        newpath = self.dirctrl.GetPath()
        self.dirctrl.CollapsePath(self.oldpath)
        self.dirctrl.ExpandPath(newpath)
        self.oldpath = newpath
    #-------------------------------------------------------------------------------------------

    def GetPath(self):
        return self.pname
    #-------------------------------------------------------------------------------------------

    def GetFilePath(self):
        return self.dirctrl.GetFilePath()
    #-------------------------------------------------------------------------------------------

    def GetSaveFilePath(self):
        return os.path.join(self.statctrl.GetLabel()[:-2], self.dynctrl.GetValue())
    #-------------------------------------------------------------------------------------------

    def onButtonNew(self, event = None):
        stattxt = self.statctrl.GetLabel()
        if stattxt.endswith(SEPSPACE):
            stattxt = stattxt[:-2]
        stattxt = stattxt.rstrip()
        if os.path.isdir(stattxt):
            self.newpressed = True
            self.Bind(wx.EVT_TEXT_ENTER, self.onTextEnter, id = self.ID_TEXTCTRL)
            self.hitreturn.SetLabel('enter new folder name and hit return to accept')
            if not stattxt.endswith(SEPSPACE) and stattxt != os.sep:
                stattxt += SEPSPACE
                self.statctrl.SetLabel(stattxt)
            self.dynctrl.Clear()
            self.dynctrl.SetEditable(True)
            self.dynctrl.SetBackgroundColour(wx.NullColour)
            self.dynctrl.SetFocus()
            self.Layout()
    #-------------------------------------------------------------------------------------------
        
    def onTextEnter(self, event = None):
        self.dynctrl.SetEditable(False)
        self.hitreturn.SetLabel('')
        newpath = os.path.join(self.dirctrl.GetPath(), self.dynctrl.GetValue())
        if os.path.exists(newpath):
            dlg = wx.MessageDialog(self, "Directory, '%s'\nalready exists. Choose a different name." % (newpath), TITLE + ' - Create new dir', wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()
            return False
            
        self.newpressed = False
        self.Unbind(wx.EVT_TEXT_ENTER, id = self.ID_TEXTCTRL)
        try:
            os.mkdir(newpath)
            self.dirctrl.ReCreateTree()
            self.dirctrl.ExpandPath(newpath)
            self.pname = self.dirctrl.GetPath()
            self.dynctrl.SetValue(self.fname)            
            return True
        except:
            dlg = wx.MessageDialog(self, "Sorry, '%s'\ncould not be created. Check your permissions." % (newpath), TITLE + ' - Create new dir', wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()
            return False
    #-------------------------------------------------------------------------------------------

    def windowPosition(self):
        windowrect = self.GetRect()
        self.config['Window']['dirposx'] = windowrect[0]
        self.config['Window']['dirposy'] = windowrect[1]
        self.config['Window']['dirsizex'] = windowrect[2]
        self.config['Window']['dirsizey'] = windowrect[3]
    #-------------------------------------------------------------------------------------------
                
    def onButtonCopy(self, event):
        if self.newpressed:
            if not self.onTextEnter():
                return
        self.pname = self.dirctrl.GetPath()
        self.windowPosition()
        self.EndModal(wx.ID_OPEN)
    #-------------------------------------------------------------------------------------------
    
    def onButtonSave(self, event):
        self.Unbind(wx.EVT_TEXT_ENTER, id = self.ID_TEXTCTRL)
        self.pname = self.statctrl.GetLabel()
        self.windowPosition()
        self.EndModal(wx.ID_SAVE)
# - class DirDialog(wx.Dialog):---------------------------------------------------------------
