#!/usr/bin/env python
#-*- coding: UTF-8 -*-

import wx
import os.path
from . import mumadir
from   .mumamisc import *

class ImportDialog(wx.Dialog):
    
    ID_START = wx.NewId()
    ID_BTN_TITLE = wx.NewId()
    ID_BTN_PLACE = wx.NewId()
    
    def __init__(self, parent, writeToLogwin, config, database):        
        """   """
        windowpos = (wx.Point(int(config['Window']['impposx']),
                              int(config['Window']['impposy'])))
        windowsize = (wx.Size(int(config['Window']['impsizex']),
                              int(config['Window']['impsizey'])))        
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - Import csv to database',
                           pos = windowpos, size = windowsize,
                           style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.Bind(wx.EVT_CLOSE, self.onButtonClose)
        self.parent = parent
        self.writeToLogwin = writeToLogwin
        self.config = config
        btnadj = int(parent.config['Misc']['norm'] == 'False')
        self.database = database
        background_color = wx.SystemSettings.GetColour(READONLY_COLOR)
        self.titlecsvpath = ''
        self.placecsvpath = ''

        sizer = wx.BoxSizer(wx.VERTICAL)
        sb = wx.StaticBox(self, -1, ' Import status messages ')
        row1 = wx.StaticBoxSizer(sb, wx.VERTICAL)       
        self.tccsv = wx.TextCtrl(self, -1,
                                 style = wx.TE_RICH | wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_DONTWRAP)      
        if LINUX and wx.VERSION_STRING < '3': self.tccsv.Enable(False) 
        self.tccsv.SetBackgroundColour(background_color)
        row1.Add(self.tccsv, 2, wx.EXPAND | wx.ALL, GAP)

        msgtext = wx.StaticText(self, -1, 'the coloured lines caused import problems (i.e. doubled ids, format errors, ...)')
        font = wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        msgtext.SetFont(font)
        row1.Add(msgtext, 0, wx.GROW | wx.ALL, GAP)

        sb = wx.StaticBox(self, -1, ' Select Title and / or Place csv file(s) ')
        row2 = wx.StaticBoxSizer(sb, wx.VERTICAL)
        fgsz = wx.FlexGridSizer(2, 3, 0, 0)
        fgsz.AddGrowableCol(1)
        st = wx.StaticText(self, -1, 'Title csv file:')
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.ctrlti = wx.TextCtrl(self, -1, '', style = wx.TE_READONLY)
        self.ctrlti.SetBackgroundColour(background_color)
        fgsz.Add(self.ctrlti, 0, wx.ALL | wx.GROW, GAP)
        self.btnti = wx.Button(self, self.ID_BTN_TITLE, 'Browse') 
        self.Bind(wx.EVT_BUTTON, self.onTitle, id = self.ID_BTN_TITLE)
        fgsz.Add(self.btnti, 0, wx.ALL, GAP)
        
        st = wx.StaticText(self, -1, 'Place csv file:')
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.ctrlpl = wx.TextCtrl(self, -1, '', style = wx.TE_READONLY)
        self.ctrlpl.SetBackgroundColour(background_color)
        fgsz.Add(self.ctrlpl, 0, wx.ALL | wx.GROW, GAP)
        self.btnpl = wx.Button(self, self.ID_BTN_PLACE, 'Browse') 
        self.Bind(wx.EVT_BUTTON, self.onPlace, id = self.ID_BTN_PLACE)
        fgsz.Add(self.btnpl, 0, wx.ALL, GAP)
        row2.Add(fgsz, 1, wx.EXPAND | wx.ALL, GAP)
                
        self.butstart = wx.Button(self, self.ID_START, '&Start Import')
        butclose = wx.Button(self, wx.ID_CLOSE)
        self.Bind(wx.EVT_BUTTON, self.onButtonStart, id = self.ID_START)
        self.Bind(wx.EVT_BUTTON, self.onButtonClose, id = wx.ID_CLOSE)     

        row3 = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: row3.AddStretchSpacer()
        row3.Add(self.butstart, btnadj, wx.GROW | wx.RIGHT, GAP)
        row3.Add(butclose, btnadj, wx.GROW | wx.LEFT, GAP)

        sizer.Add(row1, 3, wx.GROW | LRT, BORDER)
        sizer.Add(row2, 0, wx.GROW | LRT, BORDER)
        sizer.Add(row3, 0, wx.GROW | wx.ALL, BORDER)
        self.SetSizer(sizer)
        self.SetMinSize(wx.Size(520, 500))
    #-------------------------------------------------------------------------------------------

    def onTitle(self, event):
        dlg = mumadir.DirDialog(self, TITLE + ' - open titles as csv file', self.config, 
                                self.config['Misc']['titlecsv'], mode = 'openf', filters = WILDCSV)
        if dlg.ShowModal() == wx.ID_OPEN:            
            self.titlecsvpath = dlg.GetFilePath()            
            self.config['Misc']['titlecsv'] = self.titlecsvpath
            self.ctrlti.SetValue(self.titlecsvpath)
        dlg.Destroy() 
    #-------------------------------------------------------------------------------------------        

    def onPlace(self, event):
        dlg = mumadir.DirDialog(self, TITLE + ' - open places as csv file', self.config, 
                                self.config['Misc']['placecsv'], mode = 'openf', filters = WILDCSV)
        if dlg.ShowModal() == wx.ID_OPEN:
            self.placecsvpath = dlg.GetFilePath()
            self.config['Misc']['placecsv'] = self.placecsvpath
            self.ctrlpl.SetValue(self.placecsvpath)
        dlg.Destroy() 
    #-------------------------------------------------------------------------------------------        

    def onButtonStart(self, event):
        self.butstart.Enable(False)
        titlefpath = os.path.normpath(self.titlecsvpath)
        importcount = 0
        errorcount = 0
        if len(titlefpath):
            if os.path.isfile(titlefpath):
                infile = open(titlefpath, 'rt')
                self.tccsv.WriteText("starting import of '%s'\n" % (titlefpath))
                self.writeToLogwin("Import - importing titles file '%s'" % (titlefpath))
                wx.SafeYield(self)
                lines = infile.readlines()
                infile.close()
                for rdline in lines:
                    line = rdline.replace('\n', '')
                    try:
                        splitline = line.split(';')
                        if len(splitline[4]):                
                            if not '-' in splitline[4]:
                                date = splitline[4].split('.')
                                if len(date[2]) == 2:
                                    if date[2] < '40':
                                        splitline[4] = '20%s-%s-%s' % (date[2], date[1], date[0])
                                    else:
                                        splitline[4] = '19%s-%s-%s' % (date[2], date[1], date[0])
                                else:
                                    splitline[4] = '%s-%s-%s' % (date[2], date[1], date[0])
                        for i in range(7):
                            splitline[i] = splitline[i].strip()
                        record = tuple(splitline)
                        self.database.putTitleRecord(record)
                        importcount += 1
                    except:
                        start = self.tccsv.GetInsertionPoint()
                        self.tccsv.WriteText(rdline)
                        stop = self.tccsv.GetLastPosition()
                        self.tccsv.SetStyle(start, stop, wx.TextAttr(RED))
                        self.tccsv.ShowPosition(stop)
                        wx.SafeYield(self)
                        errorcount += 1
                   
                self.database.commitTransaction()
                self.tccsv.WriteText('done - %d imported records, %d errors\n\n' % (importcount, errorcount))
                self.writeToLogwin("Import - %d imported records, %d errors" % (importcount, errorcount))
                self.tccsv.ShowPosition(-1)
        
        placefpath = os.path.normpath(self.placecsvpath)
        importcount = 0
        errorcount = 0
        if len(placefpath):
            if os.path.isfile(placefpath):
                infile = open(placefpath, 'rt')
                self.tccsv.WriteText("starting import of '%s'\n" % (placefpath))
                self.writeToLogwin("Import - importing places file '%s'" % (placefpath))
                wx.SafeYield(self)
                lines = infile.readlines()
                infile.close()
                for rdline in lines:
                    try:
                        splitline = rdline.split(';')
                        if not '-' in splitline[1]:
                            date = splitline[1].split('.')
                            if len(date[2]) == 2:
                                if date[2] < '40':
                                    splitline[4] = '20%s-%s-%s' % (date[2], date[1], date[0])
                                else:
                                    splitline[4] = '19%s-%s-%s' % (date[2], date[1], date[0])
                            else:
                                splitline[4] = '%s-%s-%s' % (date[2], date[1], date[0])
                        for i in range(3):
                            splitline[i] = splitline[i].strip()
                        record = tuple(splitline)        
                        self.database.putPlaceRecord(record)
                        importcount += 1
                    except:
                        start = self.tccsv.GetInsertionPoint()
                        self.tccsv.WriteText(rdline)
                        stop = self.tccsv.GetLastPosition()
                        self.tccsv.SetStyle(start, stop, wx.TextAttr(RED))
                        self.tccsv.ShowPosition(-1)
                        errorcount += 1
                        wx.SafeYield(self)
                         
                self.database.commitTransaction()
                recordcount = format3P(self.database.getRecordCount(self.database.TAB_PLACE))
                self.tccsv.WriteText('done - %d imported records, %d errors\n\n' % (importcount, errorcount))
                self.writeToLogwin("Import - %d imported records, %d errors" % (importcount, errorcount))
                self.tccsv.ShowPosition(-1)
            
        self.butstart.Enable(True)
    #-------------------------------------------------------------------------------------------

    def windowPosition(self):
        windowrect = self.GetRect()
        self.config['Window']['impposx'] = windowrect[0]
        self.config['Window']['impposy'] = windowrect[1]
        self.config['Window']['impsizex'] = windowrect[2]
        self.config['Window']['impsizey'] = windowrect[3]
    #-------------------------------------------------------------------------------------------

    def onButtonClose(self, event):
        self.windowPosition()
        self.Destroy()
# - class QueriesManDialog(wx.Dialog):----------------------------------------------------------

