#!/usr/bin/env python
#-*- coding: UTF-8 -*-
 
import wx
from wx.lib.embeddedimage import PyEmbeddedImage
from .mumamisc import *

#----------------------------------------------------------------------
SmallUpArrow = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAADxJ"
    "REFUOI1jZGRiZqAEMFGke2gY8P/f3/9kGwDTjM8QnAaga8JlCG3CAJdt2MQxDCAUaOjyjKMp"
    "cRAYAABS2CPsss3BWQAAAABJRU5ErkJggg==")

#----------------------------------------------------------------------
SmallDnArrow = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABHNCSVQICAgIfAhkiAAAAEhJ"
    "REFUOI1jZGRiZqAEMFGke9QABgYGBgYWdIH///7+J6SJkYmZEacLkCUJacZqAD5DsInTLhDR"
    "bcPlKrwugGnCFy6Mo3mBAQChDgRlP4RC7wAAAABJRU5ErkJggg==")
#----------------------------------------------------------------------


class VirtualListCtrl(wx.ListCtrl): 
    
    def __init__(self, parent, id, config, noheader = False, singlesel = False):
        
        style = wx.LC_REPORT | wx.LC_VIRTUAL | wx.BORDER_SUNKEN
        if config ['ListCtrl']['grid'] == 'True':
            style |= wx.LC_HRULES | wx.LC_VRULES
        if noheader:
            style |= wx.LC_NO_HEADER
        if singlesel:
            style |= wx.LC_SINGLE_SEL
        wx.ListCtrl.__init__(self, parent, id, style = style) 

        self.imagelist = wx.ImageList(16, 16, True, 5) 
        self.sm_up = self.imagelist.Add(SmallUpArrow.GetBitmap())
        self.sm_dn = self.imagelist.Add(SmallDnArrow.GetBitmap())
        self.fileidx = self.imagelist.Add(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_OTHER, wx.Size(16, 16)))
        self.mfileidx = self.imagelist.Add(loadIcon('audiofile.png'))
        self.pfileidx = self.imagelist.Add(loadIcon('person.png'))
        self.SetImageList(self.imagelist, wx.IMAGE_LIST_SMALL)
 
        self.config = config
        self.datasource = []
        self.SetItemCount(0)
        self.setNewColor(config)
    #-------------------------------------------------------------------------------------------
    
    def getColumnText(self, index, col):
        return self.GetItem(index, col).GetText()
    #-------------------------------------------------------------------------------------------

    def OnGetItemText(self, item, col):
        return str(self.datasource[item][col])  # wx4
    #-------------------------------------------------------------------------------------------
    
    def OnGetItemAttr(self, item):
        data = self.datasource[item]
        try:
            if data[0] % 100 == 0:
                return self.attrvol
            elif data[0] in self.misslist:
                return self.attrmiss
            elif data[0] >= 990000:
                return self.attr99
            elif item % 2 == 0:
                return self.attralt
            else:
                return None
        except:
            if item % 2 == 0:
                return self.attralt
            else:
                return None
    #-------------------------------------------------------------------------------------------

    def SetColumnCount(self, columncount):
        self._columnsortflag = [0] * columncount
        self._column = -1
        for col in range(columncount):
            self.ClearColumnImage(col)
    #-------------------------------------------------------------------------------------------

    def GetSortDirection(self, col):
        if col < len(self._columnsortflag):
            return self._columnsortflag[col]
        else:
            return -1
    #-------------------------------------------------------------------------------------------

    def GetSortImages(self):
        return (self.sm_dn, self.sm_up)
    #-------------------------------------------------------------------------------------------

    def ClearImages(self):
        if self._column != -1:
            self.ClearColumnImage(self._column)
            self._column = -1
    #-------------------------------------------------------------------------------------------
        
    def UpdateImages(self, column, direction):
        oldcolumn = self._column
        if column != -1:
            self._column = column
            self._columnsortflag[column] = direction
        
            sortimages = self.GetSortImages()
            if self._column != -1 and sortimages[0] != -1:
                imgindex = sortimages[self._columnsortflag[self._column]]
                if oldcolumn != -1:
                    self.ClearColumnImage(oldcolumn)
                self.SetColumnImage(self._column, imgindex)
    #-------------------------------------------------------------------------------------------

    def OnGetItemImage(self, item):
        try:
            if self.datasource[item][0] % 100 == 0:
                return self.fileidx
            else:
                return self.mfileidx
        except:
            return self.pfileidx
    #-------------------------------------------------------------------------------------------
    
    def setNewColor(self, config):
        self.attralt = wx.ListItemAttr()
        collist = config['Misc']['altcolor']
        altcol = (int(collist[0]), int(collist[1]), int(collist[2]))  
        self.attralt.SetBackgroundColour(altcol)
        self.attrvol = wx.ListItemAttr()
        collist = config['Misc']['hlcolor']
        hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))        
        self.attrvol.SetBackgroundColour(hlcolor)
        collist = config['Misc']['99color']
        color99 = (int(collist[0]), int(collist[1]), int(collist[2]))        
        self.attr99 = wx.ListItemAttr()
        self.attr99.SetBackgroundColour(color99)
        collist = config['Misc']['misscolor']
        colormiss = (int(collist[0]), int(collist[1]), int(collist[2]))  
        self.attrmiss = wx.ListItemAttr()
        self.attrmiss.SetBackgroundColour(colormiss)
    #-------------------------------------------------------------------------------------------

    def setDataSource(self, datasource, itemcount, misslist = []):
        self.DeleteAllItems()
        self.isalbum = False
        self.datasource = datasource
        self.SetItemCount(itemcount)
        self.misslist = misslist
# - class VirtualListCtrl(wx.ListCtrl):--------------------------------------------------------
